
from flask import Flask, jsonify, current_app, Blueprint 

from flask_restful import Resource, Api, reqparse
from flask_swagger import swagger 

import python_settings

import libamplifier_lib as amp

from amplifiersetting import LockCFunction
api = Api()
api_point = Blueprint('api', __name__)

parser = reqparse.RequestParser()
parser.add_argument('room_name')

class RoomName(Resource):
    @LockCFunction
    def get(self):
        """
        Get the room name 
        
        ---
        tags:
          - Room
        responses:
          200:
            description: Returns the room name
        """
        room_name = python_settings.amplifier_setting.room
        return jsonify({'room_name': room_name})
    @LockCFunction
    def post(self):
        """
        Create a new user
        ---
        swagger_from_file: website/swagger/room_name_post.yml
        """
        args = parser.parse_args()
        room_name = args['room_name']
        python_settings.amplifier_setting.set_room(room_name)
        return jsonify({'room_name': room_name})


class FatalErrors(Resource):
    
    @LockCFunction
    def get(self):
        """
        Get a dict of the currently asserted fatal errors
        
        ---
        tags:
          - Error
        responses:
          200:
            description: Returns the fatal errors with a timestamp of when they happened
        """
        fatal_errors = python_settings.amplifier_setting.get_fatal_error_history()
        return {'fatal_errors': fatal_errors}


class TransErrors(Resource):
    @LockCFunction
    def get(self):
        """
        Get a dict of the currently asserted transient errors
        
        ---
        tags:
          - Error
        responses:
          200:
            description: Returns the transient errors that are currently asserted 
        """
        trans_errors = python_settings.amplifier_setting.get_trans_history()
        return jsonify({'trans_errors': trans_errors})

class UpdateAvailable(Resource):
    @LockCFunction
    def get(self):
        """
        Get if an update is available 
        ---
        tags:
          - Update
        responses:
          200:
            description: Returns a boolean of if an update is available
        """
        print "GET"
        update = python_settings.amplifier_setting.get_update_available()
        return jsonify({'update_available': update})

    @LockCFunction
    def put(self):
        """
        Start an available update
        ---
        tags:
          - Update
        responses:
          200:
            description: Starts an update returns whether the update was successful
        """
        print "PUT"
        update = python_settings.amplifier_setting.start_update()
        return jsonify({'updated': update})


class UpdateProgress(Resource):

    @LockCFunction
    def get(self):
        """
        The progress of an update in progress
        ---
        tags:
          - Update
        responses:
          200:
            description: Returns a boolean of if an update is available
        """
        update_progress = python_settings.amplifier_setting.amp_setting.GetUpdatePercent()
        return jsonify({'update_progress': update_progress})


class UpdateInProgress(Resource):

    @LockCFunction
    def get(self):
        """
        Whether the amp is currently updating
        ---
        tags:
          - Update
        responses:
          200:
            description: Returns a boolean of if an update is in progress
        """
        update = python_settings.amplifier_setting.amp_setting.GetUpdateInProgress()
        return jsonify({'update_in_progress': update})


class ModelName(Resource):
    @LockCFunction
    def get(self):
        """
        Get the amplifiers model name
        ---
        tags:
          - Model
        responses:
          200:
            description: Returns a string of the model name
        """
        
        model_name = python_settings.amplifier_setting.get_model_name()
        return jsonify({'model_name': model_name})

class PowerSwitch(Resource):
    @LockCFunction
    def get(self):
        """
        Get the power state of the amplifier. 
        ---
        tags:
          - Power
        responses:
          200:
            description: Returns the current power state of the amplifier True for On, False for Off
        """
        power_state = python_settings.amplifier_setting.amp_setting.GetPowerOn()
        return jsonify({'power_state':  power_state})

    def put(self):
        """
        Toggle the power state of the amplifier. 
        ---
        tags:
          - Power
        responses:
          200:
            description: Returns the new power state of the amplifier True for On, False for Off
        """
        power_state = python_settings.amplifier_setting.amp_setting.GetPowerOn()
        if not power_state:
            python_settings.amplifier_setting.power_on()
        else:
            python_settings.amplifier_setting.power_off()
        power_state = python_settings.amplifier_setting.amp_setting.GetPowerOn()
        return jsonify({'power_state':  power_state})
        

class Temperature(Resource):
    @LockCFunction
    def get(self):
        """
        Get the amplifiers current temperature 
        ---
        tags:
          - Temperature
        responses:
          200:
            description: Returns the temperature in Celcius 
        """
        temp = python_settings.amplifier_setting.amp_setting.GetTemperature()
        return jsonify({'temperature':"%.1f" % temp})

class Logs(Resource):
    @LockCFunction
    def put(self):
        """
        Upload the current logs of the amplifier
        ---
        tags:
          - Logs
        responses:
          200:
            description: Upload the current logs 
        """
        amp.uploadLogFiles()
        return jsonify({'upload': True})


class ModelVersion(Resource):
    @LockCFunction
    def get(self):
        """
        Get  the current model version of the unit
        ---
        tags:
          - Version 
        responses:
          200:
            description:  Output model version
        """
        version = python_settings.amplifier_setting.amp_setting.GetModelVersion()
        return jsonify({'model_version': version})
                

class Version(Resource):
    @LockCFunction
    def get(self):
        """
        Get current software version 
        ---
        tags:
          - Version 
        responses:
          200:
            description:  Output software version
        """
        version = amp.GetSoftwareVersion()
        return jsonify({'software_version': version})
      
class MainsState(Resource):
    @LockCFunction
    def get(self):
        """
        Get current state of the mains pin 
        ---
        tags:
          - ADC Mains 
        responses:
          200:
            description:  Output a message indicating the state of the mains pin
        """
        current_state = python_settings.amplifier_setting.amp_setting.GetMainsError()
        if current_state == amp.mains_state.Over_Critical or current_state == amp.mains_state.Over_Warning:
            message = "Mains Voltage High"
        elif current_state == amp.mains_state.Under_Critical:
            message = "Mains Voltage Low"
        elif current_state == amp.mains_state.Clear:
            message = "Mains Voltage OK"
        else:
            message = "Mains Voltage in unknown state"
        return jsonify({'mains_state': message})


class Mains(Resource):
    @LockCFunction
    def get(self):
        """
        Get current analog voltagc on the mains pin 
        ---
        tags:
          - ADC Mains 
        responses:
          200:
            description:  Output the bits given on the mains pin
        """
        mains_bits = python_settings.amplifier_setting.get_mains_value()
        return jsonify({'mains': mains_bits})

class SerialNumber(Resource):
    @LockCFunction
    def get(self):
        """
       Get the unit's serial number
       ---
       tags:
         - Global
       responses:
         200:
           description: Returns the serial number
        """
        serial_num = amp.GetSerialNumber()
        return jsonify({'serial_number': serial_num})


        serial_num = amp.GetSerialNumber()
        return {'serial_number': serial_num}
@api_point.route("/spec")
def spec():
    swag= swagger(current_app, from_file_keyword='swagger_from_file')
    swag['info']['title'] = 'BoulderAmplifier'
    return jsonify(swag)


def init_rest_api(app):
    api.add_resource(RoomName, '/api/room/room_name')
    api.add_resource(FatalErrors, '/api/error/fatal_errors')
    api.add_resource(TransErrors, '/api/error/transient_errors')
    api.add_resource(UpdateAvailable, '/api/update/update_available')
    api.add_resource(UpdateInProgress, '/api/update/update_in_progress')
    api.add_resource(UpdateProgress, '/api/update/update_progress')
    api.add_resource(ModelName, '/api/model/model_name')
    api.add_resource(PowerSwitch, '/api/power/power_switch')
    api.add_resource(Temperature, '/api/temperature/temperature')
    api.add_resource(Logs, '/api/log/upload')
    api.add_resource(Version, '/api/version/version')
    api.add_resource(ModelVersion, '/api/version/model_version')
    api.add_resource(Mains, '/api/mains/mains')
    api.add_resource(MainsState, '/api/mains/mains_state')
    api.add_resource(SerialNumber, '/api/global/serial_number')
    api.init_app(app)

def register_headers(app):
    @app.after_request
    def after_request(response):
        response.headers.add('Access-Control-Allow-Origin','*')
        response.headers.add('Access-Control-Allow-Headers', "Authorization, Content-Type")
        response.headers.add('Access-Control-Expose-Headers', "Authorization")
        response.headers.add('Access-Control-Allow-Methods', "GET, POST, PUT, DELETE, OPTIONS")
        response.headers.add('Access-Control-Allow-Credentials', "true")
        response.headers.add('Access-Control-Max-Age', 60 * 60 * 24 * 20)
        return response


if __name__ == '__main__':
    app = Flask(__name__)
    app.register_blueprint(api_point, url_prefix='/api')
    init_rest_api(app)
    register_headers(app)
    app.run(debug=True)
